/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package tools.descartes.dql.connector.nop;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import tools.descartes.dql.connector.AbstractOSGiQueryConnector;
import tools.descartes.dql.connector.QueryProgressListener;
import tools.descartes.dql.core.engine.util.DQLLogger;
import tools.descartes.dql.models.mapping.domain.DomainFactory;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Probe;

public class AbstractNopQueryConnector extends AbstractOSGiQueryConnector {
	public final static Integer NUMBER_OF_ELEMENTS = 2;
	protected Logger log = DQLLogger.getLogger(this.getClass().getName());

	private List<QueryProgressListener> listeners = new ArrayList<QueryProgressListener>();

	protected Probe createOnlineSeriesProbe(final DomainFactory domainFactory) {
		final Probe probe = domainFactory.createProbe();

		probe.setMetricName("onlineSeriesMetric");
		log.debug("Appending probe " + probe.getMetricName());

		return probe;
	}
	
	protected Probe createListOfContinuousSeriesProbe(final DomainFactory domainFactory) {
		final Probe probe = domainFactory.createProbe();

		probe.setMetricName("listOfContinuousSeriesMetric");
		log.debug("Appending probe " + probe.getMetricName());

		return probe;
	}

	protected Probe createContinuousSeriesProbe(final DomainFactory domainFactory) {
		final Probe probe = domainFactory.createProbe();

		probe.setMetricName("continuousSeriesMetric");
		log.debug("Appending probe " + probe.getMetricName());

		return probe;
	}

	protected Probe createDiscreteSeriesProbe(final DomainFactory domainFactory) {
		final Probe probe = domainFactory.createProbe();

		probe.setMetricName("discreteSeriesMetric");
		log.debug("Appending probe " + probe.getMetricName());

		return probe;
	}

	protected Probe createStringProbe(final DomainFactory domainFactory) {
		final Probe probe = domainFactory.createProbe();

		probe.setMetricName("stringMetric");
		log.debug("Appending probe " + probe.getMetricName());

		return probe;
	}

	protected Probe createDecimalProbe(final DomainFactory domainFactory, final int number) {
		final Probe probe = domainFactory.createProbe();

		probe.setMetricName("decimalMetric" + number);
		log.debug("Appending probe " + probe.getMetricName());

		return probe;
	}

	protected Entity createEntity(final DomainFactory domainFactory, int j, String entityType) {
		final Entity service = domainFactory.createEntity();

		service.setIdentifier("res" + j);
		service.setEntityType(entityType);

		return service;
	}

	@Override
	public void addQueryProgressListener(QueryProgressListener listener) {
		listeners.add(listener);
	}

	public void updateSubTask(String subTask) {
		for (QueryProgressListener listener : listeners) {
			listener.updateSubTask(subTask);
		}
	}

	public void updateProgress(float progressInPercent) {
		for (QueryProgressListener listener : listeners) {
			listener.updateProgress(progressInPercent);
		}
	}

	public void updateRemainingTime(int seconds) {
		for (QueryProgressListener listener : listeners) {
			listener.updateRemainingTime(seconds);
		}
	}

	protected void fakeProgress() {
		try {
			Thread.sleep(100);
			for (int i = 0; i <= 100; i++) {
				updateProgress(i * 1.0f);
				if (i % 25 == 0) {
					updateSubTask("task " + ((i / 25) + 1));
				}
				if (i % 10 == 0) {
					updateRemainingTime(10 - (i / 10));
				}
				Thread.sleep(10);
			}
		} catch (InterruptedException e) {
		}
	}
}
