package tools.descartes.dql.core.engine.interpreter;

import tools.descartes.dql.core.engine.query.QueryExecutionEngine;
import tools.descartes.dql.core.engine.query.QueryExecutionEngineImpl;
import tools.descartes.dql.lang.descartesQL.DescartesQL;
import tools.descartes.dql.lang.descartesQL.DescartesQLFactory;
import tools.descartes.dql.lang.descartesQL.EntityReference;
import tools.descartes.dql.lang.descartesQL.ForClause;
import tools.descartes.dql.lang.descartesQL.ListEntitiesQuery;
import tools.descartes.dql.lang.descartesQL.ListMetricsQuery;
import tools.descartes.dql.lang.descartesQL.ListQuery;
import tools.descartes.dql.lang.descartesQL.SelectQuery;
import tools.descartes.dql.lang.descartesQL.UsingClause;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

public class DefaultProvider {

	private DescartesQLFactory dqlFactory = DescartesQLFactory.eINSTANCE;
	private EntityMapping statTypes;
	private EntityMapping entityTypes;
	private UsingClause usingClause;
	private ForClause forClause;

	/*
	 * Since SelectQuery and ListQuery don't have a common ancestor, we need to
	 * use a little workaround
	 */
	public DefaultProvider(SelectQuery selectQuery, ListQuery listQuery) {
		if (selectQuery != null) {
			this.usingClause = selectQuery.getUsingClause();
			this.forClause = selectQuery.getForClause();
		} else {
			this.usingClause = listQuery.getUsingClause();
		}
	}

	public String getDefaultStatType(String entityRef, String metricRef2) {
		for (Entity e : getStatTypes().getEntities())
			if (e.getAlias().equals(entityRef))
				for (Probe p : e.getProbes())
					if (p.getMetricName().equals(metricRef2))
						if (p.getStatTypes().isEmpty())
							return "DEFAULT";
						else
							return p.getStatTypes().get(0).getTypeName();
		throw new IllegalStateException("There is no Metric " + entityRef + "." + metricRef2);
	}

	private EntityMapping getEntityTypes() {
		if (entityTypes == null)
			initializeEntityTypes();
		return entityTypes;
	}

	private void initializeEntityTypes() {
		DescartesQL entityQuery = dqlFactory.createDescartesQL();
		ListQuery listQuery = dqlFactory.createListQuery();
		entityQuery.setQuery(listQuery);
		listQuery.setUsingClause(usingClause);
		ListEntitiesQuery listMetricsQuery = dqlFactory.createListEntitiesQuery();
		listQuery.setQueryType(listMetricsQuery);
		listMetricsQuery.setMode("LIST ENTITIES");

		QueryExecutionEngine qee = new QueryExecutionEngineImpl();
		this.entityTypes = qee.execute(entityQuery).get(0);
	}

	private EntityMapping getStatTypes() {
		if (statTypes == null)
			initializeStatTypes();
		return statTypes;
	}

	private void initializeStatTypes() {
		DescartesQL metricsQuery = dqlFactory.createDescartesQL();
		ListQuery listQuery = dqlFactory.createListQuery();
		metricsQuery.setQuery(listQuery);
		listQuery.setUsingClause(usingClause);
		ListMetricsQuery listMetricsQuery = dqlFactory.createListMetricsQuery();
		listQuery.setQueryType(listMetricsQuery);
		listMetricsQuery.setMode("LIST METRICS");
		listMetricsQuery.setEntityReferenceClause(forClause.getEntityReferenceClause());

		QueryExecutionEngine qee = new QueryExecutionEngineImpl();
		this.statTypes = qee.execute(metricsQuery).get(0);
	}

	public String lookUpEntityType(EntityReference ref) {
		String identifier = ref.getIdentifier();

		String type = null;
		for (Entity e : getEntityTypes().getEntities())
			if (e.getIdentifier().equals(identifier))
				if (type == null)
					type = e.getEntityType();
				else
					throw new IllegalStateException("There is more than one Entity with the identifier " + identifier
							+ ", please specify the entityType");

		if (type == null)
			throw new IllegalStateException("There is no Entity with the identifier " + identifier);
		return type;
	}
}
