/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.engine.util;

import static tools.descartes.dql.core.engine.util.QueryParserHelper.intervalToProperties;
import static tools.descartes.dql.core.engine.util.QueryParserHelper.toFlatConfigurationPropertiesClause;
import static tools.descartes.dql.core.engine.util.QueryParserHelper.valueToProperties;

import java.util.Properties;

import org.eclipse.emf.ecore.EObject;

import tools.descartes.dql.lang.descartesQL.ConfigurationPropertiesClause;
import tools.descartes.dql.lang.descartesQL.DoFVariationClause;
import tools.descartes.dql.lang.descartesQL.IntervalVariationClause;
import tools.descartes.dql.lang.descartesQL.ValueVariationClause;

public class DoFHelper {

	public static Properties processVariationClause(final EObject variationClause, Properties variationProperties) {
		if (variationClause instanceof ConfigurationPropertiesClause) {
			variationProperties = toFlatConfigurationPropertiesClause((ConfigurationPropertiesClause) variationClause);
		}

		if (variationClause instanceof DoFVariationClause) {
			final EObject variation = ((DoFVariationClause) variationClause).getVariation();

			if (variation instanceof IntervalVariationClause) {
				final IntervalVariationClause interval = (IntervalVariationClause) variation;
				variationProperties = intervalToProperties(interval);
			}

			if (variation instanceof ValueVariationClause) {
				final ValueVariationClause value = (ValueVariationClause) variation;
				variationProperties = valueToProperties(value);
			}
		}

		return variationProperties;
	}
}
