/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */


package tools.descartes.dql.core.engine.whatif;

import tools.descartes.dql.lang.descartesQL.FilterigRule;
import tools.descartes.dql.lang.descartesQL.FilteringClause;

public class WhatIfFilterFactory {

	public static WhatIfFilter parseFilter(FilteringClause clause) {
		if (clause.getFilteringStrategy() != null)
			return parseFilter(clause.getFilteringStrategy().getFilteringStrategy());
		if (clause.getFilterungRule() != null) {
			return parseFilter(clause.getFilterungRule());
		}
		throw new IllegalStateException("Illegal filtering strategy");
	}

	private static WhatIfFilter parseFilter(FilterigRule filteringRule) {

		int filteringInt = filteringRule.getFilteringValue().getIntValue();
		double filteringDouble = filteringRule.getFilteringValue().getDoubleValue();

		if (filteringInt != 0 && filteringDouble == 0.0)
			if (filteringRule.getModifier() != null && filteringRule.getModifier().equals("%"))
				return new DoubleFilter((double) filteringInt, true);
			else
				return new DoubleFilter((double) filteringInt, false);
		if (filteringDouble != 0.0 && filteringInt == 0)
			if (filteringRule.getModifier() != null && filteringRule.getModifier().equals("%"))
				return new DoubleFilter(filteringDouble, true);
			else
				return new DoubleFilter(filteringDouble, false);

		throw new IllegalArgumentException("No valid filter found");

	}

	private static WhatIfFilter parseFilter(String filteringStrategy) {
		if (filteringStrategy.matches("ENTITY .+"))
			return new EntityFilter(filteringStrategy.substring(7));
		if (filteringStrategy.matches("METRIC .+"))
			return new MetricFilter(filteringStrategy.substring(7));
		if (filteringStrategy.matches("ONLY 0") || filteringStrategy.matches("0 ONLY") || filteringStrategy.matches("0") || filteringStrategy.matches("0 VALUES")
				|| filteringStrategy.matches("DEFAULT"))
			return new ZeroFilter();
		return null;
	}
}

