/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.platform.eclipse.views;

import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.ALIAS_POS;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.ALIGNMENT;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.HEADINGS;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.IDENTIFIER_POS;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.METRIC_POS;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.STATTYPE_POS;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.TYPE_POS;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.VALID_POS;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.VALUE_POS;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.WEIGHTS;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.WIDTHS;

import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.part.ViewPart;

import tools.descartes.dql.models.mapping.mapping.EntityMapping;

public class QueryResultTableViewer extends ViewPart {
	/**
	 * The ID of the view as specified by the extension.
	 */
	public static final String ID = "tools.descartes.dql.core.platform.eclipse.views.QueryResultTableViewer";

	private TableViewer viewer;

	private EntityMapping queryResult;

	private Composite composite;

	private TableColumnLayout layout;

	/**
	 * The constructor.
	 */
	public QueryResultTableViewer() {
	}

	private void createColumn(final TableViewer viewer, final TableColumnLayout layout, final int position) {
		final TableViewerColumn tableViewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn tableColumn = tableViewerColumn.getColumn();
		final String heading = HEADINGS[position];
		final int alignment = ALIGNMENT[position];
		final int weight = WEIGHTS[position];
		final int minimumWidth = WIDTHS[position];
		final ColumnWeightData columnWeight = new ColumnWeightData(weight, minimumWidth, true);

		layout.setColumnData(tableColumn, columnWeight);

		tableColumn.setText(heading);
		tableColumn.setToolTipText(heading);
		tableColumn.setResizable(true);
		tableColumn.setAlignment(alignment);
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 */
	@Override
	public void createPartControl(final Composite parent) {
		composite = new Composite(parent, SWT.NONE);
		final int style = SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL;
		viewer = new TableViewer(composite, style);
		layout = new TableColumnLayout();

		composite.setLayout(layout);

		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		createColumn(viewer, layout, TYPE_POS);
		createColumn(viewer, layout, ALIAS_POS);
		createColumn(viewer, layout, IDENTIFIER_POS);
		createColumn(viewer, layout, METRIC_POS);
		createColumn(viewer, layout, STATTYPE_POS);
		createColumn(viewer, layout, VALID_POS);
		createColumn(viewer, layout, VALUE_POS);

		viewer.setContentProvider(new ViewContentProvider(this));
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.setComparator(new TableRowBeanComparator());

		if (queryResult != null) {
			viewer.setInput(queryResult);
		} else {
			viewer.setInput(getViewSite());
		}

		// Create the help context id for the viewer's control
		hookContextMenu();
	}

	private void hookContextMenu() {
		final MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		final Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	public void refresh() {
		viewer.refresh();
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	public void setQueryResult(final EntityMapping queryResult) {
		this.queryResult = queryResult;
		viewer.setInput(queryResult);
	}

	public void setQueryTitle(final String title) {
		super.setPartName(title);
	}
}
