/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
/*
 * generated by Xtext
 */
package tools.descartes.dql.lang.ui.contentassist;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.xtext.Assignment;
import org.eclipse.xtext.RuleCall;
import org.eclipse.xtext.ui.editor.contentassist.ContentAssistContext;
import org.eclipse.xtext.ui.editor.contentassist.ICompletionProposalAcceptor;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;

import tools.descartes.dql.core.engine.query.QueryExecutionEngine;
import tools.descartes.dql.core.engine.query.QueryExecutionEngineImpl;
import tools.descartes.dql.core.registry.ConnectorRegistry;
import tools.descartes.dql.lang.descartesQL.AliasClause;
import tools.descartes.dql.lang.descartesQL.DescartesQL;
import tools.descartes.dql.lang.descartesQL.DescartesQLFactory;
import tools.descartes.dql.lang.descartesQL.EntityReference;
import tools.descartes.dql.lang.descartesQL.EntityReferenceClause;
import tools.descartes.dql.lang.descartesQL.ForClause;
import tools.descartes.dql.lang.descartesQL.ListEntitiesQuery;
import tools.descartes.dql.lang.descartesQL.ListMetricsQuery;
import tools.descartes.dql.lang.descartesQL.ListQuery;
import tools.descartes.dql.lang.descartesQL.MetricClause;
import tools.descartes.dql.lang.descartesQL.MetricReference;
import tools.descartes.dql.lang.descartesQL.SelectQuery;
import tools.descartes.dql.lang.descartesQL.UsingClause;
import tools.descartes.dql.lang.descartesQL.impl.MetricReferenceImpl;
import tools.descartes.dql.lang.ui.internal.DescartesQLActivator;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.domain.StatType;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

/**
 * see
 * http://www.eclipse.org/Xtext/documentation/latest/xtext.html#contentAssist on
 * how to customize content assistant
 */
public class DescartesQLProposalProvider extends AbstractDescartesQLProposalProvider {
	private final BundleContext context = DescartesQLActivator.getInstance().getBundle().getBundleContext();
	private ConnectorRegistry registry = null;
	private final DescartesQLFactory dqlFactory = DescartesQLFactory.eINSTANCE;

	@Override
	public void completeMetricReference_StatTypeReference(EObject model, Assignment assignment,
			ContentAssistContext context, ICompletionProposalAcceptor acceptor) {
		final EObject container = context.getCurrentModel().eContainer().eContainer().eContainer();

		MetricReferenceImpl mri = (MetricReferenceImpl) context.getCurrentModel();
		String entityName = mri.getEntityReference();
		String metricName = mri.getMetricReference();

		if (!(container instanceof SelectQuery)) {
			return;
		}

		final SelectQuery selectQuery = (SelectQuery) container;
		UsingClause usingClause = EcoreUtil.copy(selectQuery.getUsingClause());
		final ForClause forClause = selectQuery.getForClause();

		if (usingClause == null || forClause == null || forClause.getEntityReferenceClause() == null)
			return;

		// Build Query
		Map<String, String> aliases = new HashMap<String, String>();
		for (EntityReference er : forClause.getEntityReferenceClause().getEntityReferences())
			aliases.put(er.getAlias().getAlias(), er.getIdentifier());
		final DescartesQL metricsQuery = dqlFactory.createDescartesQL();
		final ListQuery listQuery = dqlFactory.createListQuery();
		metricsQuery.setQuery(listQuery);
		listQuery.setUsingClause(usingClause);
		final ListMetricsQuery listMetricsQuery = dqlFactory.createListMetricsQuery();
		listQuery.setQueryType(listMetricsQuery);
		listMetricsQuery.setMode("LIST METRICS");
		listMetricsQuery.setEntityReferenceClause(forClause.getEntityReferenceClause());

		final QueryExecutionEngine qee = new QueryExecutionEngineImpl();
		final EntityMapping result = qee.execute(metricsQuery).get(0);

		// Finde entity/metric pair
		for (Entity e : result.getEntities())
			if (e.getAlias().equals(entityName))
				for (Probe p : e.getProbes())
					if (p.getMetricName().equals(metricName))
						for (StatType st : p.getStatTypes())
							acceptor.accept(createCompletionProposal(st.getTypeName(), context));
	}

	@Override
	public void complete_ModelFamily(final EObject model, final RuleCall ruleCall, final ContentAssistContext context,
			final ICompletionProposalAcceptor acceptor) {
		super.complete_ModelFamily(model, ruleCall, context, acceptor);

		for (final String modelFamily : retrieveSupportedModelFamilies()) {
			acceptor.accept(createCompletionProposal(modelFamily, context));
		}
	}

	@Override
	public void completeEntityReference_Identifier(final EObject model, final Assignment assignment,
			final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
		super.completeEntityReference_Identifier(model, assignment, context, acceptor);

		final EObject rootModel = context.getRootModel();
		final DescartesQL queryModel = (DescartesQL) rootModel;
		final EObject query = queryModel.getQuery();
		HashMap<String, Set<String>> identifiers = null;
		UsingClause usingClause = null;

		if (query instanceof SelectQuery) {
			final SelectQuery selectQuery = (SelectQuery) query;
			usingClause = EcoreUtil.copy(selectQuery.getUsingClause());
		} else {

			return;
		}

		if (usingClause != null) {
			final DescartesQL root = dqlFactory.createDescartesQL();

			final ListQuery list = dqlFactory.createListQuery();
			root.setQuery(list);
			list.setUsingClause(usingClause);

			final ListEntitiesQuery listEntities = dqlFactory.createListEntitiesQuery();
			list.setQueryType(listEntities);
			listEntities.setMode("ENTITIES");

			identifiers = submitListEntitiesQuery(root);
		}

		if (identifiers == null) {
			return;
		}

		final EObject previousModel = context.getPreviousModel();

		// As RESOURCE or SERVICE is usually in front of an identifier, try to
		// filter for a specific entity type to shorten the result list
		if (previousModel instanceof EntityReference) {
			final EntityReference previous = (EntityReference) previousModel;
			final String previousType = previous.getType();

			if (previousType != null) {
				for (final String identifier : identifiers.get(previousType)) {
					provideEntityContentAssistEntry(context, acceptor, previousType, identifier);
				}

				return;
			}
		}

		// Fall back: Process all entities
		for (Entry<String, Set<String>> entry : identifiers.entrySet()) {
			final String availableType = entry.getKey();
			for (final String identifier : identifiers.get(availableType)) {
				provideEntityContentAssistEntry(context, acceptor, availableType, identifier);
			}
		}

	}

	@Override
	public void completeMetricReference_EntityReference(final EObject model, final Assignment assignment,
			final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
		super.completeMetricReference_EntityReference(model, assignment, context, acceptor);

		final String prefix = context.getPrefix();
		final EObject container = context.getCurrentModel().eContainer();

		if (!(container instanceof SelectQuery)) {
			return;
		}

		final SelectQuery selectQuery = (SelectQuery) container;
		final ForClause forClause = selectQuery.getForClause();

		if (forClause == null) {
			return;
		}

		final Set<String> aliases = filterPrefixes(retrieveAliases(forClause), prefix);

		propose(context, acceptor, aliases, prefix);
	}

	@Override
	public void completeMetricReference_MetricReference(final EObject model, final Assignment assignment,
			final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
		super.completeMetricReference_MetricReference(model, assignment, context, acceptor);

		final EObject rootModel = context.getRootModel();
		final DescartesQL queryModel = (DescartesQL) rootModel;
		final EObject query = queryModel.getQuery();

		if (!(query instanceof SelectQuery)) {
			return;
		}

		final SelectQuery selectQuery = (SelectQuery) query;
		final String prefix = context.getPrefix();
		final EObject container = context.getCurrentModel().eContainer();
		final MetricClause metricClause = (MetricClause) container;
		final MetricReference metricReference = metricClause.getMetricReference();

		final String entity = metricReference.getEntityReference();
		metricReference.getMetricReference();

		final EntityReference entityReference = searchForClause(selectQuery.getForClause(), entity);

		if (entityReference == null) {
			return;
		}

		UsingClause usingClause = selectQuery.getUsingClause();

		if (usingClause == null) {
			return;
		}

		usingClause = EcoreUtil.copy(usingClause);

		final DescartesQL metricsQuery = dqlFactory.createDescartesQL();

		final ListQuery listQuery = dqlFactory.createListQuery();
		metricsQuery.setQuery(listQuery);
		listQuery.setUsingClause(usingClause);

		final ListMetricsQuery listMetricsQuery = dqlFactory.createListMetricsQuery();
		listQuery.setQueryType(listMetricsQuery);
		listMetricsQuery.setMode("LIST METRICS");

		final EntityReferenceClause eRef = dqlFactory.createEntityReferenceClause();
		listMetricsQuery.setEntityReferenceClause(eRef);
		eRef.getEntityReferences().add(entityReference);

		final Set<String> metrics = filterPrefixes(submitListMetricsQuery(entityReference, metricsQuery), prefix);

		propose(context, acceptor, metrics, prefix);
	}

	private synchronized void createConnectorRegistryInstance() {
		if (registry != null) {
			return;
		}

		registry = lookupConnectorRegistryInstance();
	}

	private Set<String> filterPrefixes(final Set<String> elements, final String prefix) {
		final Set<String> filteredElements = new HashSet<String>(elements.size());

		if (prefix.isEmpty()) {
			filteredElements.addAll(elements);
		} else {
			for (final String element : elements) {
				if (element.startsWith(prefix)) {
					filteredElements.add(element);
				}
			}
		}

		return filteredElements;
	}

	// Stolen from AbstractQueryInterpreter
	private ConnectorRegistry lookupConnectorRegistryInstance() {
		final ServiceReference<ConnectorRegistry> registryServiceRef = context
				.getServiceReference(ConnectorRegistry.class);
		return context.getService(registryServiceRef);
	}

	private void propose(final ContentAssistContext context, final ICompletionProposalAcceptor acceptor,
			final Set<String> proposals, final String prefix) {
		for (final String alias : proposals) {
			final String proposal = alias.substring(prefix.length());
			acceptor.accept(createCompletionProposal(proposal, alias, null, context));
		}
	}

	private void provideEntityContentAssistEntry(final ContentAssistContext context,
			final ICompletionProposalAcceptor acceptor, final String availableType, final String identifier) {
		final String proposal = "'" + identifier + "'";
		final String displayString = availableType + ": " + identifier;
		acceptor.accept(createCompletionProposal(proposal, displayString, null, context));
	}

	private Set<String> retrieveAliases(final ForClause forClause) {
		final HashSet<String> aliases = new HashSet<String>();

		final EntityReferenceClause entityReferenceClause = forClause.getEntityReferenceClause();

		if (entityReferenceClause == null) {
			return aliases;
		}

		for (final EntityReference e : entityReferenceClause.getEntityReferences()) {
			final AliasClause alias = e.getAlias();

			if (alias != null) {
				aliases.add(alias.getAlias());
			} else {
				aliases.add(e.getIdentifier());
			}
		}

		return aliases;
	}

	private Set<String> retrieveSupportedModelFamilies() {
		createConnectorRegistryInstance();

		return registry.getRegisteredConnectorNames();
	}

	private EntityReference searchForClause(final ForClause forClause, final String entity) {
		if (forClause == null) {
			return null;
		}

		final EntityReferenceClause entityReferenceClause = forClause.getEntityReferenceClause();

		if (entityReferenceClause == null) {
			return null;
		}

		for (final EntityReference e : entityReferenceClause.getEntityReferences()) {
			final String identifier = e.getIdentifier();
			final AliasClause alias = e.getAlias();

			// Found the same identifier
			if (entity.equals(identifier)) {
				return EcoreUtil.copy(e);
			}

			// Found the same alias
			if (alias != null) {
				if (entity.equals(alias.getAlias())) {
					return EcoreUtil.copy(e);
				}
			}

		}

		return null;
	}

	private HashMap<String, Set<String>> submitListEntitiesQuery(final DescartesQL root) {
		final HashMap<String, Set<String>> identifiers = new HashMap<String, Set<String>>(2);
		final QueryExecutionEngine qee = new QueryExecutionEngineImpl();
		final List<EntityMapping> results = qee.execute(root);

		if (qee.isValidQuery()) {
			final EntityMapping result = results.get(0);
			final EList<Entity> resources = result.getEntities();

			for (final Entity r : resources) {
				String type = r.getEntityType();
				if (!identifiers.containsKey(type)) {
					final Set<String> resourceIdentifiers = new HashSet<String>(resources.size());
					resourceIdentifiers.add(r.getIdentifier());
					identifiers.put(type, resourceIdentifiers);
				} else {
					identifiers.get(type).add(r.getIdentifier());
				}
			}

		}

		return identifiers;
	}

	private Set<String> submitListMetricsQuery(final EntityReference entityReference, final DescartesQL metricsQuery) {
		final Set<String> metrics = new HashSet<String>();
		final QueryExecutionEngine qee = new QueryExecutionEngineImpl();
		final List<EntityMapping> results = qee.execute(metricsQuery);

		if (!qee.isValidQuery()) {
			return metrics;
		}

		final EntityMapping result = results.get(0);
		final String type = entityReference.getType();
		Entity resultEntity = null;

		for (Entity e : result.getEntities())
			if ((e.getEntityType() == null && type == null) || e.getEntityType().equals(type))
				resultEntity = e;

		if (resultEntity == null)
			return metrics;

		for (final Probe p : resultEntity.getProbes()) {
			final String retrievedMetricName = p.getMetricName();
			metrics.add(retrievedMetricName);
		}

		return metrics;
	}
}
