package tools.descartes.pavo;

/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import org.jfree.ui.RefineryUtilities;

//import tools.descartes.pavo.types.DifferenceChart;
//import tools.descartes.pavo.types.HeatMap;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.domain.StatType;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;
import tools.descartes.pavo.controller.VisualizationController;

/**
 * main class of the pavo library. Creates one frame with one Tab for each
 * Resource received by the EntityMapping just create an object of this class
 * with the EntityMapping and a title, to run pavo
 * 
 * @author Maximilian König
 */

public class GraphicsEngine {

	private JFrame mainFrame = new JFrame();
	private EntityMapping currentMapping;
	private JTabbedPane tabbedPane = new JTabbedPane();

	public GraphicsEngine(String title, EntityMapping mappingNeu) {

		// HeatMap demo = new HeatMap("Block Chart Demo");
		// demo.pack();
		// RefineryUtilities.centerFrameOnScreen(demo);
		// demo.setVisible(true);

		currentMapping = mappingNeu;
		mainFrame.setTitle(title);
		mainFrame.setBackground(Color.WHITE);
		mainFrame.add(tabbedPane);
		interpreteEntityMapping(currentMapping, tabbedPane);
		mainFrame.pack();
		mainFrame.setVisible(true);
		mainFrame.setResizable(false);
		mainFrame.toFront();
		RefineryUtilities.centerFrameOnScreen(mainFrame);
	}

	/**
	 * Method to interprete the EntityMapping received by DQL and merges
	 * diagrams creates tabs for each resource/entities for this resource
	 * 
	 * @param EntityMapping
	 *            mapping: current EntityMapping to be visualized
	 * @param JTabbedPane
	 *            tabPane: Tab Pane for all different resources.
	 * 
	 */
	public static void interpreteEntityMapping(EntityMapping mapping, JTabbedPane tabPane) {

		List<String> metricNames = new LinkedList<String>();
		List<List<String>> resourceNames = new LinkedList<List<String>>();
		List<List<StatType>> listStatTypeToMetric = new LinkedList<List<StatType>>();

		for (Entity e : mapping.getEntities()) {
			
			for (Probe p : e.getProbes()) {
				
				for (StatType s : p.getStatTypes()) {
					
					boolean found = false;
					String newName = p.getMetricName() +"."+ s.getTypeName();
					for (int i = 0; i < metricNames.size(); i++) {

						if (metricNames.get(i).equals(newName)) {
							listStatTypeToMetric.get(i).add(s);
							resourceNames.get(i).add(e.getAlias());
							found = true;
						}
					}
					if (found == false) {
						metricNames.add(newName);
						LinkedList<StatType> help = new LinkedList<StatType>();
						help.add(s);
						LinkedList<String> help2 = new LinkedList<String>();
						help2.add(e.getAlias());
						listStatTypeToMetric.add(help);
						resourceNames.add(help2);
					}
				}
			}
		}

			for (int i = 0; i < metricNames.size(); i++) {

				tabPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
				// Put querytext on top
				String queryText = "<html><body>"
					+ mapping.getQueryAsString().substring(0, mapping.getQueryAsString().indexOf("FOR"))
						+ "<br>"
					+ mapping.getQueryAsString().substring(mapping.getQueryAsString().indexOf("FOR"),
								mapping.getQueryAsString().indexOf("USING"))
						+ "<br>" + mapping.getQueryAsString().substring(mapping.getQueryAsString().indexOf("USING"),
								mapping.getQueryAsString().length())
						+ "</body></html>";
				JLabel querytext = new JLabel(queryText);
				querytext.setFont(new Font("Arial", Font.BOLD, 15));
				querytext.setBackground(Color.WHITE);
				querytext.setOpaque(true);
				querytext.setSize(new Dimension(tabPane.getWidth(), 200));
				JPanel resourcePanel = new JPanel(new BorderLayout());
				resourcePanel.add(querytext, BorderLayout.NORTH);
				// create the visualizer for each entity and add the panel to
				// the
				// tabbed pane
				VisualizationController engine = new VisualizationController(listStatTypeToMetric.get(i),
						resourceNames.get(i), metricNames.get(i));

//				engine.interpreteResource();
				resourcePanel.add(Box.createRigidArea(new Dimension(tabPane.getWidth(), 20)), BorderLayout.EAST);
				resourcePanel.add(engine.getView().getCompletePanel(), BorderLayout.SOUTH);
				tabPane.addTab(metricNames.get(i), resourcePanel);
			}

		}
}