/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.controller;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import tools.descartes.dql.models.mapping.domain.ConfidenceInterval;
import tools.descartes.dql.models.mapping.domain.ContinuousSeriesResult;
import tools.descartes.dql.models.mapping.domain.DecimalResult;
import tools.descartes.dql.models.mapping.domain.DiscreteSeriesResult;
import tools.descartes.dql.models.mapping.domain.OnlineResult;
import tools.descartes.dql.models.mapping.domain.Result;
import tools.descartes.dql.models.mapping.domain.ResultList;
import tools.descartes.dql.models.mapping.domain.SeriesResultElement;
import tools.descartes.dql.models.mapping.domain.StatType;
import tools.descartes.dql.models.mapping.domain.StringResult;
import tools.descartes.pavo.result.AbstractQuantitativeResult;
import tools.descartes.pavo.result.ResultContainerPavo;
import tools.descartes.pavo.result.SeriesResultPavo;
import tools.descartes.pavo.result.StringResultPavo;
import tools.descartes.pavo.result.ValueResultPavo;

/**
 * class to visualize all probes of one resource into one pane. These panes are
 * merged within the graphicsengine
 */

// Die Marker bei BarConfidence funktionieren nicht

public class TransformEntityMappingToResult {

	

	

	public static ResultContainerPavo transformEntityMapping(List<StatType> currentProbes, List<String> resourcesToProbes, String metricName) {
		ResultContainerPavo result = new ResultContainerPavo();
		
		int j = 0;
		for (StatType currentProbe : currentProbes) {
			
			
				
			if (currentProbe.getResult() instanceof DecimalResult) {

				AbstractQuantitativeResult r = new ValueResultPavo(resourcesToProbes.get(j), metricName, "StatType", "Scale", ((DecimalResult) currentProbe.getResult()).getValue().doubleValue());
				result.addResult(r);
				//				view.setStringResult("Value of " + ((DecimalResult) currentProbe).getValue().toString() + " for "
//						+ metricName + " for Resource " + );

			} else if (currentProbe.getResult() instanceof StringResult) {

				AbstractQuantitativeResult r = new StringResultPavo(resourcesToProbes.get(j), metricName, "StatType", "Scale", ((StringResult) currentProbe).getValue().toString());
				 result.addResult(r);
//				view.setStringResult(((StringResult) currentProbe.getResult()).getValue().toString() + " for "
//						+ metricName + " for Resource " + resourcesToProbes.get(j));
//				result.getStringResult().add(((StringResult) currentProbe).getValue().toString());
			}

			else if (currentProbe.getResult() instanceof DiscreteSeriesResult) {
				
				
				EList<SeriesResultElement> list = ((DiscreteSeriesResult) currentProbe.getResult()).getElements();
				List<Double> x = new ArrayList<Double>();
				List<Double> y = new ArrayList<Double>();
				for (SeriesResultElement a : list) {
					x.add(a.getX().doubleValue());
					y.add(a.getY().doubleValue());
				}

				List<ConfidenceInterval> confidences = new ArrayList<ConfidenceInterval>();
				for (SeriesResultElement a : list) {
					confidences.add(a.getConfidenceInterval());
				}

//				result.setxAxisLabel(((DiscreteSeriesResult) currentProbe.getResult()).getXAxisDescription());
//				result.setyAxisLabel(((DiscreteSeriesResult) currentProbe.getResult()).getYAxisDescription());
//				result.getxValues().add(x);
//				result.getyValues().add(y);
//				result.getSeriesNames().add(resourcesToProbes.get(j));
//				result.getConfidences().add(confidences);
				
				AbstractQuantitativeResult r = new SeriesResultPavo(resourcesToProbes.get(j), metricName, "StatType", "Scale", x, y, false, "XSCALE", "YSCALE", confidences);
				 result.addResult(r);
			}

			else if (currentProbe.getResult() instanceof ContinuousSeriesResult) {

//				shownDiagram = EnumDiagramtype.LINECHART;
				EList<SeriesResultElement> list = ((ContinuousSeriesResult) currentProbe.getResult()).getElements();
				List<Double> x = new ArrayList<Double>();
				List<Double> y = new ArrayList<Double>();
				for (SeriesResultElement a : list) {
					x.add(a.getX().doubleValue());
					y.add(a.getY().doubleValue());
				}

				List<ConfidenceInterval> confidences = new ArrayList<ConfidenceInterval>();
				for (SeriesResultElement a : list) {
					confidences.add(a.getConfidenceInterval());
				}

//				result.setxAxisLabel(((ContinuousSeriesResult) currentProbe.getResult()).getXAxisDescription());
//				result.setyAxisLabel(((ContinuousSeriesResult) currentProbe.getResult()).getYAxisDescription());
//				result.getxValues().add(x);
//				result.getyValues().add(y);
//				result.getSeriesNames().add(resourcesToProbes.get(j));
//				result.getConfidences().add(confidences);
				
				AbstractQuantitativeResult r = new SeriesResultPavo(resourcesToProbes.get(j), metricName, "StatType", "Scale", x, y, true, "XSCALE", "YSCALE", confidences);
				 result.addResult(r);
				 
			} else if (currentProbe.getResult() instanceof ResultList<?>) {

//				shownDiagram = EnumDiagramtype.LINECHART;

				List<List<Double>> xWerte = new ArrayList<List<Double>>();
				List<List<Double>> yWerte = new ArrayList<List<Double>>();
				List<List<ConfidenceInterval>> confindencesN = new ArrayList<List<ConfidenceInterval>>();
				List<String> nameSeries = new ArrayList<String>();
//				@SuppressWarnings("unchecked")
//				String nameX = ((ResultList<ContinuousSeriesResult>) currentProbe.getResult()).getResults().get(0)
//						.getXAxisDescription();
//				@SuppressWarnings("unchecked")
//				String nameY = ((ResultList<ContinuousSeriesResult>) currentProbe.getResult()).getResults().get(0)
//						.getYAxisDescription();
				int i = 1;

				for (Result c : ((ResultList<?>) currentProbe.getResult()).getResults()) {

					EList<SeriesResultElement> list = ((ContinuousSeriesResult) c).getElements();
					List<Double> x = new ArrayList<Double>();
					List<Double> y = new ArrayList<Double>();
					List<ConfidenceInterval> confidences = new ArrayList<ConfidenceInterval>();
					for (SeriesResultElement a : list) {
						x.add(a.getX().doubleValue());
						y.add(a.getY().doubleValue());
						confidences.add(a.getConfidenceInterval());
					}
					xWerte.add(x);
					yWerte.add(y);
					confindencesN.add(confidences);
					i++;

				}

//				result.setxAxisLabel(nameX);
//				result.setyAxisLabel(nameY);
//				result.getxValues().addAll(xWerte);
//				result.getyValues().addAll(yWerte);
//				result.getSeriesNames().add(resourcesToProbes.get(j));
//				result.getConfidences().addAll(confindencesN);
				
				for(int a = 0; a < i; a++)
				{
					result.addResult(new SeriesResultPavo(resourcesToProbes.get(j), metricName, "StatType", "Scale", xWerte.get(a), yWerte.get(a), true, "XSCALE", "YSCALE", confindencesN.get(a)));
				}
						

			} else if (currentProbe.getResult() instanceof OnlineResult) {

//				shownDiagram = EnumDiagramtype.LINECHART;
//				view.getChangeButton().setVisible(false);
//				view.getDiagramSwitchTypes().setVisible(false);
//				numberOnlineSeries = j;
//
//				currentResult = ((OnlineResult) currentProbe.getResult()).getResult();
//				result.setxAxisLabel("x");
//				result.setyAxisLabel("y");
//
//				result.getSeriesNames().add(resourcesToProbes.get(j));
//				result.getxValues().add(new ArrayList<Double>());
//				result.getyValues().add(new ArrayList<Double>());
//				result.getConfidences().add(new ArrayList<ConfidenceInterval>());
//				System.out.println(result.getSeriesNames());
//
//				Adapter adapter = new AdapterImpl() {
//					public synchronized void notifyChanged(Notification notification) {
//
//						EList<SeriesResultElement> c = ((ContinuousSeriesResult) currentResult).getElements();
//						SeriesResultElement element = c.get(DotsShown);
//						result.getxValues().get(numberOnlineSeries).add(element.getX().doubleValue());
//						result.getyValues().get(numberOnlineSeries).add(element.getY().doubleValue());
//						result.getConfidences().get(numberOnlineSeries).add(element.getConfidenceInterval());
//						draw();
//						DotsShown++;
//
//					}
//				};
//				currentResult.eAdapters().add(adapter);
			}
			j++;
		}
		return result;

	}

	}