/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.controller;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.swing.JFileChooser;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.Marker;
import org.jfree.chart.plot.ValueMarker;
import org.jfree.ui.RectangleAnchor;

import tools.descartes.dql.models.mapping.domain.StatType;
import tools.descartes.pavo.types.BarChart;
import tools.descartes.pavo.types.BoxPlot;
import tools.descartes.pavo.types.DifferenceChart;
import tools.descartes.pavo.types.HeatMap;
import tools.descartes.pavo.types.IVisualizationType2;
import tools.descartes.pavo.types.LineChart;
import tools.descartes.pavo.types.PieChart;
import tools.descartes.pavo.view.ExportDialogFrame;
import tools.descartes.pavo.view.View;
import tools.descartes.pavo.helper.MathHelper;
import tools.descartes.pavo.result.*;

/**
 * class to visualize all probes of one resource into one pane. These panes are
 * merged within the graphicsengine
 */

// Die Marker bei BarConfidence funktionieren nicht

public class VisualizationController {

	View view;
	private boolean min = false;
	private boolean max = false;
	private boolean mean = false;
	private List<ValueMarker> minMarkers = new ArrayList<ValueMarker>();
	private List<ValueMarker> maxMarkers = new ArrayList<ValueMarker>();
	private List<ValueMarker> meanMarkers = new ArrayList<ValueMarker>();


	private JFreeChart chart = null;
	private JFileChooser c = new JFileChooser();

	// Variables for OnlinePlot
//	private int DotsShown = 0;
//	private Result currentResult;
//	int numberOnlineSeries;

	private ResultContainerPavo result = new ResultContainerPavo(); 

	private List<Class<? extends IVisualizationType2>> AllDiagramClasses = new LinkedList<Class<? extends IVisualizationType2>>();
	private Class<? extends IVisualizationType2> currentShownDiagramClass;

	
	public VisualizationController(List<StatType> r, List<String> resources, String metricNameN) {

		view = new View();
		this.setActionListener();

		result = TransformEntityMappingToResult.transformEntityMapping(r, resources, metricNameN);
		
		
//		List<ClassLoader> classLoadersList = new LinkedList<ClassLoader>();
//		classLoadersList.add(ClasspathHelper.contextClassLoader());
//		classLoadersList.add(ClasspathHelper.staticClassLoader());
//
//		Reflections reflections = new Reflections(new ConfigurationBuilder()
//		    .setScanners(new SubTypesScanner(false /* don't exclude Object.class */), new ResourcesScanner())
//		    .setUrls(ClasspathHelper.forClassLoader(classLoadersList.toArray(new ClassLoader[0])))
//		    .filterInputsBy(new FilterBuilder().include(FilterBuilder.prefix(BarChart.class.getPackage().toString()))));
//		
//		Set<Class<?>> classes = reflections.getSubTypesOf(Object.class);
//		System.out.println(classes.size());
//		for(Class c : classes)
//		{
//			System.out.println(c.getName());
//		}
		
		AllDiagramClasses.add(BarChart.class);
		AllDiagramClasses.add(LineChart.class);
		AllDiagramClasses.add(BoxPlot.class);
		AllDiagramClasses.add(DifferenceChart.class);
		AllDiagramClasses.add(HeatMap.class);
		AllDiagramClasses.add(PieChart.class);
		
	
		draw();
		checkButtons();
		fillComboBoxToDiagram();
		
	}
	

	public void addMin() {

		for (AbstractQuantitativeResult r : result.getResults()) {
			
			double value = MathHelper.getMin(((SeriesResultPavo) r).getyvalues());
			String res = "Minimum of " + r.getQueryableElement();
 			boolean found = false;

			for (ValueMarker m : minMarkers) {
				if (m.getValue() == value) {
					m.setLabel(m.getLabel() + " and " + res);
					found = true;
				}
			}

			if (found == false) {
				ValueMarker marker = new ValueMarker(value);
				marker.setPaint(Color.green);
				marker.setLabel(res);
				marker.setLabelAnchor(RectangleAnchor.CENTER);
				chart.getXYPlot().addRangeMarker(marker);
				minMarkers.add(marker);
			}

		}
	}

	public void addMax() {
		for (AbstractQuantitativeResult r : result.getResults()) {
			
			double value = MathHelper.getMax(((SeriesResultPavo) r).getyvalues());
			String res = "Maximum of " + r.getQueryableElement();
 			boolean found = false;

			for (ValueMarker m : maxMarkers) {
				if (m.getValue() == value) {
					m.setLabel(m.getLabel() + " and " + res);
					found = true;
				}
			}

			if (found == false) {
				ValueMarker marker = new ValueMarker(value);
				marker.setPaint(Color.orange);
				marker.setLabel(res);
				marker.setLabelAnchor(RectangleAnchor.CENTER);
				chart.getXYPlot().addRangeMarker(marker);
				maxMarkers.add(marker);
			}

		}

	}

	public void addMean() {
		for (AbstractQuantitativeResult r : result.getResults()) {
			
			double value = MathHelper.getMean(((SeriesResultPavo) r).getyvalues());
			String res = "Mean of " + r.getQueryableElement();
 			boolean found = false;

			for (ValueMarker m : meanMarkers) {
				if (m.getValue() == value) {
					m.setLabel(m.getLabel() + " and " + res);
					found = true;
				}
			}

			if (found == false) {
				ValueMarker marker = new ValueMarker(value);
				marker.setPaint(Color.blue);
				marker.setLabel(res);
				marker.setLabelAnchor(RectangleAnchor.CENTER);
				chart.getXYPlot().addRangeMarker(marker);
				meanMarkers.add(marker);
			}

		}
	}

	public void setActionListener() {

		view.setActionListenerMinRadio(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				if (min) {
					for (Marker m : minMarkers) {
						chart.getXYPlot().removeRangeMarker(m);
					}
					min = false;
				} else {
					addMin();
					min = true;
				}

			}
		});

		view.setActionListenerMaxRadio(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				if (max) {
					for (Marker m : maxMarkers) {
						chart.getXYPlot().removeRangeMarker(m);
					}
					max = false;
				} else {
					addMax();
					max = true;
				}

			}
		});

		view.setActionListenerMeanRadio(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				if (mean) {
					for (Marker m : meanMarkers) {
						chart.getXYPlot().removeRangeMarker(m);
					}
					mean = false;
				} else {
					addMean();
					mean = true;
				}

			}
		});

		view.setActionListenerConfidenceRadio(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				transformDiagram();
			}
		});

		view.setActionListenerChangeButton(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				transformDiagram();

			}
		});

		view.setActionListenerExportButton(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				exportDiagram();
			}
		});
	}
	
	public void switchToConfidence()
	{
		
		System.out.println("CurrentClass"+currentShownDiagramClass.getName());
		System.out.println(view.getConfidenceRadio().isSelected());
		Object helpObject = null;
		try {
			helpObject = currentShownDiagramClass.getDeclaredConstructor(ResultContainerPavo.class).newInstance(result);
			
		} catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException | NoSuchMethodException | SecurityException e) {
			e.printStackTrace();
		}
		
		if(view.getConfidenceRadio().isSelected())
		{
			Method visualizeMethodConfidence = null;
			try {
				visualizeMethodConfidence = currentShownDiagramClass.getMethod("visualizeConfidence");
			} catch (NoSuchMethodException | SecurityException e) {
				e.printStackTrace();
			}
			try {
				chart =  (JFreeChart) visualizeMethodConfidence.invoke(helpObject);
				view.updateChart(chart);
				return;
			} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
				e.printStackTrace();
			}
		}
		else
		{
			Method visualizeMethod = null;
			try {
				visualizeMethod = currentShownDiagramClass.getMethod("visualize");
			} catch (NoSuchMethodException | SecurityException e) {
				e.printStackTrace();
			}
			try {
				chart =  (JFreeChart) visualizeMethod.invoke(helpObject);
				view.updateChart(chart);
				return;
			} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
				e.printStackTrace();
			}
		}

				
	}
	
	

	public View getView() {
		return view;
	}

	public void setView(View view) {
		this.view = view;
	}

	private void exportDiagram() {
		int a = c.showSaveDialog(null);
		if (a == JFileChooser.CANCEL_OPTION) {
			System.out.println("Canceled Save Action!");
		} else {
			final File file = c.getSelectedFile();
			String name = file.getPath();
			final File fileToUse;

			if (name.length() < 3 || !name.substring(name.length() - 3, name.length()).equals("png")) {
				fileToUse = new File(name + ".png");
			} else {
				fileToUse = file;
			}

			double c = (double) (view.getChartPanel().getHeight());
			double b = (double) (view.getChartPanel().getWidth());
			double faktorforWandH = c / b;

			final ExportDialogFrame select = new ExportDialogFrame(faktorforWandH);

			select.addWindowListener(new WindowListener() {

				@Override
				public void windowOpened(WindowEvent e) {
				}

				@Override
				public void windowIconified(WindowEvent e) {
				}

				@Override
				public void windowDeiconified(WindowEvent e) {
				}

				@Override
				public void windowDeactivated(WindowEvent e) {
				}

				@Override
				public void windowClosing(WindowEvent e) {
				}

				@Override
				public void windowClosed(WindowEvent e) {
					try {
						if (select.getSuccess() == true) {
							ChartUtilities.saveChartAsPNG(fileToUse, chart, select.getWidthFromText(),
									select.getHeightFromText());
						}

					} catch (IOException e1) {
					}

				}

				@Override
				public void windowActivated(WindowEvent e) {
				}
			});
		}

	}


	public void checkButtons() {
		
		Object helpObject = null;
		try {
			helpObject = currentShownDiagramClass.getDeclaredConstructor(ResultContainerPavo.class).newInstance(result);
		} catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException
				| NoSuchMethodException | SecurityException e) {
			e.printStackTrace();
		}
		Method canConfidenceMethod = null;
		try {
			canConfidenceMethod = currentShownDiagramClass.getMethod("supportsConfidenceInterval");
		} catch (NoSuchMethodException | SecurityException e) {
			e.printStackTrace();
		}
		boolean MethodReturn = false;
		try {
			MethodReturn =  (boolean) canConfidenceMethod.invoke(helpObject);
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			e.printStackTrace();
		}
//		System.out.println("canConfidence returned: " + MethodReturn);
		view.getConfidenceRadio().setEnabled(MethodReturn);
		
		
		
		Method canAnalyses = null;
		try {
			canAnalyses = currentShownDiagramClass.getMethod("supportsAnalyses");
		} catch (NoSuchMethodException | SecurityException e) {
			e.printStackTrace();
		}
		try {
			MethodReturn =  (boolean) canAnalyses.invoke(helpObject);
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			e.printStackTrace();
		}
//		System.out.println("canAnalyses returned: " + MethodReturn);
		view.getMaxRadio().setEnabled(MethodReturn);
		view.getMinRadio().setEnabled(MethodReturn);
		view.getMeanRadio().setEnabled(MethodReturn);
	
	}

	public void draw() {
		for(Class<? extends IVisualizationType2> c : AllDiagramClasses)
		{
			
			Object helpObject = null;
			try {
				helpObject = c.getDeclaredConstructor(ResultContainerPavo.class).newInstance(result);
				
			} catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException | NoSuchMethodException | SecurityException e) {
				e.printStackTrace();
			}

			Method canVisualizeMethod = null;
			try {
				canVisualizeMethod = c.getMethod("canVisualize");
			} catch (NoSuchMethodException | SecurityException e) {
				e.printStackTrace();
			}
			boolean MethodReturn = false;
			try {
				MethodReturn =  (boolean) canVisualizeMethod.invoke(helpObject, new Object[] {});
			} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
				e.printStackTrace();
			}
//			System.out.println("canVisualize returned: " + MethodReturn);

			
			if(MethodReturn)
			{
				Method visualizeMethod = null;
				try {
					visualizeMethod = c.getMethod("visualize");
				} catch (NoSuchMethodException | SecurityException e) {
					e.printStackTrace();
				}
				try {
					chart = (JFreeChart) visualizeMethod.invoke(helpObject);
					currentShownDiagramClass = c;
					view.updateChart(chart);
					return;
				} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
					e.printStackTrace();
				}
			}
			
		}
		view.setError();
	}

	public void fillComboBoxToDiagram() {
		view.getDiagramSwitchTypes().removeAllItems();

		view.addItemToCombo(currentShownDiagramClass.getName().substring(currentShownDiagramClass.getName().lastIndexOf('.')+1, currentShownDiagramClass.getName().length()));
		
		
		for(Class<? extends IVisualizationType2> c : AllDiagramClasses)
		{
			Object helpObject = null;
			try {
				helpObject = c.getDeclaredConstructor(ResultContainerPavo.class).newInstance(result);
				
			} catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException | NoSuchMethodException | SecurityException e) {
				e.printStackTrace();
			}

			Method canVisualizeMethod = null;
			try {
				canVisualizeMethod = c.getMethod("canVisualize");
			} catch (NoSuchMethodException | SecurityException e) {
				e.printStackTrace();
			}
			boolean MethodReturn = false;
			try {
				MethodReturn =  (boolean) canVisualizeMethod.invoke(helpObject, new Object[] {});
			} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
				e.printStackTrace();
			}
//			System.out.println("canVisualize returned: " + MethodReturn);
			
			if(MethodReturn && !currentShownDiagramClass.equals(c))
			{
				view.addItemToCombo(c.getName().substring(c.getName().lastIndexOf('.')+1, c.getName().length()));
			}
	
		}
	}
	
	
	public void clearMarkerForTransform()
	{
		view.resetAnalysesRadios();
		min = false;
		max = false;
		mean = false;
		minMarkers.clear();
		maxMarkers.clear();
		meanMarkers.clear();
	}
	
	public void transformDiagram() {
		String switchType = view.getDiagramSwitchTypes().getSelectedItem().toString();
		clearMarkerForTransform();
		
		for(Class<? extends IVisualizationType2> c : AllDiagramClasses)
		{
			if(c.getName().substring(c.getName().lastIndexOf('.')+1, c.getName().length()).equals(switchType))
			{
				Object helpObject = null;
				try {
					helpObject = c.getDeclaredConstructor(ResultContainerPavo.class).newInstance(result);
					
				} catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException | NoSuchMethodException | SecurityException e) {
					e.printStackTrace();
				}

				Method visualizeMethod = null;
				if(view.getConfidenceRadio().isSelected())
				{
					try {
						visualizeMethod = c.getMethod("visualizeConfidence");
					} catch (NoSuchMethodException | SecurityException e) {
						e.printStackTrace();
					}
					try {
						chart =  (JFreeChart) visualizeMethod.invoke(helpObject, new Object[] {});
						currentShownDiagramClass = c;
						view.updateChart(chart);
						fillComboBoxToDiagram();
						checkButtons();
					} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
						e.printStackTrace();
					}
				}
				else
				{
					try {
						visualizeMethod = c.getMethod("visualize");
					} catch (NoSuchMethodException | SecurityException e) {
						e.printStackTrace();
					}
					try {
						chart =  (JFreeChart) visualizeMethod.invoke(helpObject, new Object[] {});
						currentShownDiagramClass = c;
						view.updateChart(chart);
						fillComboBoxToDiagram();
						checkButtons();
					} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
						e.printStackTrace();
					}
				}
			}
		}

		// set<classes> set1 <--- welche Klassen im Paket
		// for (IVisulalizationType viz: set)
		// if(!vis.canVisualize(result) set.remove(viz

		// IVisualizationType2 visualizationType =
		// viz.getClass.getInstance(result);
		// visualizationType.visualize()
	}
}