/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.helper;

import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

import javax.tools.JavaFileObject;
import javax.tools.StandardJavaFileManager;
import javax.tools.StandardLocation;
import javax.tools.ToolProvider;
import tools.descartes.pavo.types.*;

public class GetClassesHelper {
	   
	public List<Class> getClasses() throws Exception
	{
		return getClassesForPackage(IVisualizationType2.class.getClass().getPackage().toString());
		
	}
	
	

	private List<Class> getClassesForPackage(String pckgname) throws ClassNotFoundException {
	      // This will hold a list of directories matching the pckgname. There may be more than one if a package is split over multiple jars/paths
	      ArrayList<File> directories = new ArrayList<File>();
	      String packageToPath = pckgname.replace('.', '/');
	      try {
	          ClassLoader cld = Thread.currentThread().getContextClassLoader();
	          if (cld == null) {
	              throw new ClassNotFoundException("Can't get class loader.");
	          }

	          // Ask for all resources for the packageToPath
	          Enumeration<URL> resources = cld.getResources(packageToPath);
	          while (resources.hasMoreElements()) {
	              directories.add(new File(URLDecoder.decode(resources.nextElement().getPath(), "UTF-8")));
	          }
	      } catch (NullPointerException x) {
	          throw new ClassNotFoundException(pckgname + " does not appear to be a valid package (Null pointer exception)");
	      } catch (UnsupportedEncodingException encex) {
	          throw new ClassNotFoundException(pckgname + " does not appear to be a valid package (Unsupported encoding)");
	      } catch (IOException ioex) {
	          throw new ClassNotFoundException("IOException was thrown when trying to get all resources for " + pckgname);
	      }

	      ArrayList<Class> classes = new ArrayList<Class>();
	      // For every directoryFile identified capture all the .class files
	      while (!directories.isEmpty()){
	    	  System.out.println("BLA");
	          File directoryFile  = directories.remove(0);             
	          if (directoryFile.exists()) {
	              // Get the list of the files contained in the package
	              File[] files = directoryFile.listFiles();

	              for (File file : files) {
	            	  System.out.println("BLUBB");
	                  // we are only interested in .class files
	                  if ((file.getName().endsWith(".class")) && (!file.getName().contains("$"))) {
	                      // removes the .class extension
	                      int index = directoryFile.getPath().indexOf(packageToPath);
	                      String packagePrefix = directoryFile.getPath().substring(index).replace('/', '.');;                          
	                    try {                  
	                      String className = packagePrefix + '.' + file.getName().substring(0, file.getName().length() - 6);                            
	                      classes.add(Class.forName(className));                                
	                    } catch (NoClassDefFoundError e)
	                    {
	                      // do nothing. this class hasn't been found by the loader, and we don't care.
	                    }
	                  } else if (file.isDirectory()){ // If we got to a subdirectory
	                      directories.add(new File(file.getPath()));                          
	                  }
	              }
	          } else {
	              throw new ClassNotFoundException(pckgname + " (" + directoryFile.getPath() + ") does not appear to be a valid package");
	          }
	      }
	      return classes;
	}
}