/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 * Creates a dialog to choose the size of the exported PNG
 * 
 * @author Maximilian König
 *
 */
public class ExportDialogFrame extends JFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private JPanel contentPane;
	private JLabel header;
	private JLabel widthLabel;
	private JButton finishButton;
	private JButton cancelButton;
	private JTextField height;
	private JTextField width;
	private double factorWidthToHeight;
	private boolean saveSuccessful=false;



	/**
	 * Create the frame.
	 * @throws InterruptedException 
	 */
	public ExportDialogFrame(final double factorWidthToHeightN) {
		factorWidthToHeight = factorWidthToHeightN;
		setBounds(100, 100, 450, 300);
		contentPane = new JPanel();
		contentPane.setPreferredSize(new Dimension(400, 300));
		contentPane.setMaximumSize(new Dimension(400, 300));
		contentPane.setMinimumSize(new Dimension(400, 300));
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		contentPane.setLayout(new BorderLayout(0, 0));
		setContentPane(contentPane);
		
		header = new JLabel("Select the size of your exported Chart as PNG Image");
		header.setVerticalAlignment(SwingConstants.TOP);
		header.setMinimumSize(new Dimension(327, 70));
		header.setPreferredSize(new Dimension(327, 70));
		header.setHorizontalAlignment(SwingConstants.CENTER);
		contentPane.add(header, BorderLayout.NORTH);
		
		Box verticalBox = Box.createVerticalBox();
		contentPane.add(verticalBox, BorderLayout.CENTER);
		
		Box horizontalBox_1 = Box.createHorizontalBox();
		verticalBox.add(horizontalBox_1);
		
		widthLabel = new  JLabel("Selected Width:             ");
		horizontalBox_1.add(widthLabel);
		
		width = new JTextField("");
		width.setMaximumSize(new Dimension(200, 30));
		width.setMinimumSize(new Dimension(200, 30));
		horizontalBox_1.add(width);
		width.setPreferredSize(new Dimension(200, 30));
		
		Box horizontalBox = Box.createHorizontalBox();
		verticalBox.add(horizontalBox);
		
		JLabel lblNewLabel_1 = new JLabel("Selected Height:           ");
		horizontalBox.add(lblNewLabel_1);
		
		height = new JTextField("");
		height.setPreferredSize(new Dimension(200, 30));
		height.setMaximumSize(new Dimension(200, 30));
		height.setMinimumSize(new Dimension(200, 30));
		horizontalBox.add(height);
		height.setColumns(10);
		
		JPanel panel = new JPanel();
		contentPane.add(panel, BorderLayout.SOUTH);
		
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				close();
			}
		});
		panel.add(cancelButton);
		
		JPanel panel_1 = new JPanel();
		panel_1.setPreferredSize(new Dimension(50, 10));
		panel_1.setMinimumSize(new Dimension(50, 10));
		panel.add(panel_1);
		
		finishButton = new JButton("Finish Save");
		panel.add(finishButton);
		finishButton.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				saveSuccessful=true;
				close();
				
			}
		});
		
		width.getDocument().addDocumentListener(new DocumentListener() {
			
			@Override
			public void removeUpdate(DocumentEvent e) {
				updateHeightFromWidth();
				
			}
			
			@Override
			public void insertUpdate(DocumentEvent e) {
				updateHeightFromWidth();
				
			}
			
			@Override
			public void changedUpdate(DocumentEvent e) {
				updateHeightFromWidth();
				
			}
		});
		
		height.setEnabled(false);
		this.setVisible(true);
		 
	}
	
	
	public void updateHeightFromWidth()
	{
		finishButton.setEnabled(true);
		try {
			double x = Double.parseDouble(width.getText()) * this.factorWidthToHeight;
			int y = (int) x;
			height.setText(String.valueOf(y));
		} catch (java.lang.NumberFormatException e) {
			height.setText("ERROR");
			finishButton.setEnabled(false);
		}
	}
	
	public void close()
	{
		this.dispose();
	}
	
	public int getWidthFromText()
	{
		return new Integer(this.width.getText());
	}
	
	public int getHeightFromText()
	{
		return new Integer(this.height.getText());
	}
	
	public boolean getSuccess()
	{
		return saveSuccessful;
	}

}

