/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.view;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.LineBorder;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;

public class View  {
	
	private JPanel frame = new JPanel(new BorderLayout());
	private JPanel visualizationPanel = new JPanel();
	private JPanel buttonPanel;
	
	private JButton changeButton;
	private JButton exportButton;;
	
	private ChartPanel chartPanel = new ChartPanel(null);
	

	private JRadioButton confidenceRadio;
	
	private JRadioButton minRadio;
	private JRadioButton maxRadio;
	private JRadioButton meanRadio;
	private JComboBox<String> DiagramSwitchTypes;
	
	public View()
	{
		BoxLayout layoutForViz = new BoxLayout(visualizationPanel, BoxLayout.Y_AXIS);
		visualizationPanel.setLayout(layoutForViz);
		frame.add(visualizationPanel, BorderLayout.CENTER);
		visualizationPanel.add(chartPanel);
		frame.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		frame.setVisible(true);

		buttonPanel = initializeButtons();
		frame.add(buttonPanel, BorderLayout.SOUTH);
	}
	
	public void resetAnalysesRadios()
	{
		this.getMinRadio().setSelected(false);
		this.getMaxRadio().setSelected(false);
		this.getMeanRadio().setSelected(false);
	}
	
	
	public void setActionListenerChangeButton(ActionListener a )
	{
		changeButton.addActionListener(a);
	}
	
	public void setActionListenerExportButton(ActionListener a )
	{
		exportButton.addActionListener(a);
	}
	
	public void setActionListenerConfidenceRadio(ActionListener a )
	{
		confidenceRadio.addActionListener(a);
	}
	public void setActionListenerMinRadio(ActionListener a )
	{
		minRadio.addActionListener(a);
	}
	public void setActionListenerMaxRadio(ActionListener a )
	{
		maxRadio.addActionListener(a);
	}
	public void setActionListenerMeanRadio(ActionListener a )
	{
		meanRadio.addActionListener(a);
	}
	
	public void addItemToCombo(String s)
	{
		DiagramSwitchTypes.addItem(s);
	}
	
	/**
	 * @return the buttonPanel
	 */
	public JPanel getButtonPanel() {
		return buttonPanel;
	}


	/**
	 * @param buttonPanel the buttonPanel to set
	 */
	public void setButtonPanel(JPanel buttonPanel) {
		this.buttonPanel = buttonPanel;
	}


	/**
	 * @return the changeButton
	 */
	public JButton getChangeButton() {
		return changeButton;
	}


	/**
	 * @param changeButton the changeButton to set
	 */
	public void setChangeButton(JButton changeButton) {
		this.changeButton = changeButton;
	}


	/**
	 * @return the exportButton
	 */
	public JButton getExportButton() {
		return exportButton;
	}


	/**
	 * @param exportButton the exportButton to set
	 */
	public void setExportButton(JButton exportButton) {
		this.exportButton = exportButton;
	}


	/**
	 * @return the confidenceRadio
	 */
	public JRadioButton getConfidenceRadio() {
		return confidenceRadio;
	}


	/**
	 * @param confidenceRadio the confidenceRadio to set
	 */
	public void setConfidenceRadio(JRadioButton confidenceRadio) {
		this.confidenceRadio = confidenceRadio;
	}


	/**
	 * @return the minRadio
	 */
	public JRadioButton getMinRadio() {
		return minRadio;
	}


	/**
	 * @param minRadio the minRadio to set
	 */
	public void setMinRadio(JRadioButton minRadio) {
		this.minRadio = minRadio;
	}
	
	


	/**
	 * @return the chartPanel
	 */
	public ChartPanel getChartPanel() {
		return chartPanel;
	}


	/**
	 * @param chartPanel the chartPanel to set
	 */
	public void setChartPanel(ChartPanel chartPanel) {
		this.chartPanel = chartPanel;
	}


	/**
	 * @return the maxRadio
	 */
	public JRadioButton getMaxRadio() {
		return maxRadio;
	}


	/**
	 * @param maxRadio the maxRadio to set
	 */
	public void setMaxRadio(JRadioButton maxRadio) {
		this.maxRadio = maxRadio;
	}


	/**
	 * @return the meanRadio
	 */
	public JRadioButton getMeanRadio() {
		return meanRadio;
	}


	/**
	 * @param meanRadio the meanRadio to set
	 */
	public void setMeanRadio(JRadioButton meanRadio) {
		this.meanRadio = meanRadio;
	}


	/**
	 * @return the diagramSwitchTypes
	 */
	public JComboBox<String> getDiagramSwitchTypes() {
		return DiagramSwitchTypes;
	}


	/**
	 * @param diagramSwitchTypes the diagramSwitchTypes to set
	 */
	public void setDiagramSwitchTypes(JComboBox<String> diagramSwitchTypes) {
		DiagramSwitchTypes = diagramSwitchTypes;
	}
	
	public JPanel initializeButtons()
	{
		JPanel analysisPanel = new JPanel();
		analysisPanel.setLayout(new BoxLayout(analysisPanel, BoxLayout.Y_AXIS));
		minRadio = new JRadioButton("Show Minimum");
		maxRadio = new JRadioButton("Show Maximum");	
		meanRadio = new JRadioButton("Show Mean");
		
		analysisPanel.add(minRadio);
		analysisPanel.add(maxRadio);
		analysisPanel.add(meanRadio);

		confidenceRadio = new JRadioButton("confidence intervals");
		
		changeButton = new JButton("Switch Diagram");
		
		exportButton = new JButton("Export Diagram");
		

		DiagramSwitchTypes = new JComboBox<String>();
		JPanel buttons = new JPanel();
		buttons.add(changeButton);
		buttons.add(DiagramSwitchTypes);
		buttons.add(confidenceRadio);
		buttons.add(exportButton);
		buttons.add(analysisPanel);

		return buttons;
	}
	
	public JPanel getCompletePanel() {
		frame.validate();
		return frame;
	}
	
	public void setStringResult(String s)
	{
		JLabel labelZahl = new JLabel(s);
		labelZahl.setBackground(Color.WHITE);
		labelZahl.setOpaque(true);
		visualizationPanel.add(labelZahl);
		visualizationPanel.add(new JLabel("    "));
	}
	
	public void setError()
	{
		JLabel error = new JLabel();

		error.setText(
				"PAVO is not able to visualize this result, sorry!");
		error.setBorder(new LineBorder(Color.RED, 5, true));
		visualizationPanel.add(error, BorderLayout.CENTER);
	}
	
	public void updateChart(JFreeChart chart) {
	visualizationPanel.remove(chartPanel);
	chartPanel = new ChartPanel(chart);
	visualizationPanel.add(chartPanel, BorderLayout.CENTER);
	visualizationPanel.repaint();
	visualizationPanel.setVisible(true);

}
	
	

}